
(* SPAN - A Security Protocol Animator
   Copyright (C) 2006-2007 Yann Glouche and Thomas Genet

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.0. Last modification 1/03/07
*)
(* ..................................................................
    Module ......: Debugprinting
    Initial Date : 03 January 2005
    Compiler ....: OCAML 3.07
    Project .....: The CL-Atse Protocol Analyser, AVISPA.
 .................................................................... *)

open Types_unify;;
open Utils_unify;;
open Unifystd;;


let print_debug (str : string) : unit =
  print_string str;
  flush stdout
;;


(* returns a string describing the term ter *)
let str_term_dbg tab (ter : term) =
  let rec f inter = function
    | Var(n)           ->
	let int_type = var_type n in
	let str_type = try List.assoc int_type !localtabletype with Not_found -> string_of_int int_type in
	let str = try List.assoc n tab with Not_found -> "X" ^ (string_of_int (var_id n)) in
	( str ^ "(" ^ (string_of_int n) ^ ")" ^ ":" ^ str_type )

(*        (try List.assoc n tab   *)
(*	with Not_found -> "X"^(string_of_int n)^"_"^(string_of_int (var_id n))^":"^(string_of_int (var_type n)) 	)  *)
(*        with Not_found -> "X"^(string_of_int (var_id n))^      *)
(*                          (if var_type n=0 then "" else "_"^   *)
(*                          string_of_int (var_type n))          *)
    | Atm(n)           ->
        (try List.assoc n tab
        with Not_found -> "A"^(string_of_int (var_id n))^
                          (if var_type n=0 then "" else "_"^
                          string_of_int (var_type n)))
    | PInv(t)          -> "(" ^ (f false t) ^ ")'"
    | SCrypt(m,k)      -> "{" ^(f true m) ^ "}_"  ^ (f false k)
    | PCrypt(m,k)      -> "#{"^(f true m) ^ "}_"  ^ (f false k)
    | Uplet(h::l)      ->
        if inter then
          it_list (fun r t -> (r ^ "," ^ (f true t))) (f true h) l
        else
          "<" ^ (it_list (fun r t -> (r ^ "," ^ (f true t))) (f true h) l) ^ ">"
    | Uplet([])        -> "<>"
    | Xor(h::l)        ->
        if inter then
          it_list (fun r t -> (r ^ "+" ^ (f false t))) (f false h) l
        else
          "(" ^ (it_list (fun r t -> (r ^ "+" ^ (f false t))) (f false h) l) ^ ")"
    | Xor([])          -> "()"
    | Exp(m,l)         -> "Exp(" ^ (f false m) ^ "," ^ 
	(it_list (fun s -> function 
	            | (a,Pos) -> s ^ "*" ^ (f false a)
	            | (a,Neg) -> s ^ "/" ^ (f false a)
		 ) "1" l)   ^ ")"
  in
    f false ter(*  true for simplicity. *)
;;


let str_list_dbg table (ls : int list) : string =
  it_list (fun str n -> str^(str_term_dbg table (Var n))^" ") "" ls
;;


let str_partition_dbg table (p : t_partition) : string =
  let rec help str = function
    | c::tail -> help (str^"["^(str_list_dbg table c)^"] ") tail
    | []      -> str
  in
  if p = [] then "empty partition\n" else help "" p
;;


let str_partitions_dbg table (ps : t_partition list) : string =
  if ps = [] then "empty list of partitions\n"
  else it_list (fun str p -> str^"----\n"^(str_partition_dbg table p)^"\n") "" ps
;;


let str_pairlist_dbg table (ls : (int * int) list) : string =
  it_list (fun str (n,m) -> str^"("^(str_term_dbg table (Var n))^","^(str_term_dbg table (Var m))^") ") "" ls
;;


let str_termlist_dbg table (sep : string) (ls : term list) : string =
  it_list (fun str t -> str^(str_term_dbg table t)^sep) "" ls
;;


(* returns a string describing a substitution *)
let str_sub_ls_clot_dbg table sub (ls : t_equal) : string =
  let rec help str = function
    | (n,t)::tail ->
        let t_clot = clot_light sub t in
        help (str^(str_term_dbg table (Var n))^" <- "^(str_term_dbg table t_clot)^"\n") tail
    | []          -> str
  in
  if ls = [] then "empty subst\n" else help "" ls
;;


(* returns a string describing a substitution *)
let str_sub_clot_dbg table ((ls,_,_) as sub : t_subst) : string =
  str_sub_ls_clot_dbg table sub ls
;;


(* returns a string describing a list of substitutions *)
let str_subs_clot_dbg table (subs : t_subst list) (size : string) (u : term) (v : term) : string =
  let rec help cnt str = function
    | sub::subs ->
        let pu,sub2 = purify sub  u in
        let pv,sub2 = purify sub2 v in
        let u_clot = clot_light sub2 pu in
        let v_clot = clot_light sub2 pv in
         help (cnt+1)
        (str^"---- subst ("^(string_of_int cnt)^"/"^size^") ----\n"^
         (str_sub_clot_dbg table sub)^
         "\nu(sub) = "^(str_term_dbg table u_clot)^
         "\nv(sub) = "^(str_term_dbg table v_clot)^"\n"
         ) subs
    | []    -> str
  in
  if subs = [] then "empty subst list\n" else help 1 "" subs
;;


(* returns a string describing a substitution *)
let str_sub_ls_dbg table (ls : t_equal) : string =
  let rec help str = function
    | (n,t)::tail -> help (str^(str_term_dbg table (Var n))^" <- "^(str_term_dbg table t)^"\n") tail
    | []          -> str
  in
  if ls = [] then "empty subst\n" else help "" ls
;;


(* returns a string describing a substitution *)
let str_sub_dbg table ((ls,_,_) : t_subst) : string =
  str_sub_ls_dbg table ls
;;


(* returns a string describing a list of substitutions *)
let str_subs_dbg table (subs : t_subst list) (size : string) : string =
  let rec help cnt str = function
    | sub::subs -> help (cnt+1) (str^"---- subst ("^(string_of_int cnt)^"/"^size^") ----\n"^(str_sub_dbg table sub)) subs
    | []    -> str
  in
  if subs = [] then "empty subst list\n" else help 1 "" subs
;;


(* ***** *)

(* returns a list of string describing a substitution *)
let lst_sub_ls_dbg (ls : t_equal) : ((Types_unify.term * Types_unify.term) list) =
  let rec help str = function
    | (n,t)::tail -> help (((Var n),t)::str) tail
    | []          -> str
  in
  if ls = [] then [] else help [] ls
;;


(* returns a list of string describing a substitution *)
let lst_sub_dbg ((ls,_,_) : t_subst) :  ((Types_unify.term * Types_unify.term) list) =
  lst_sub_ls_dbg ls
;;


(* returns a list of string describing a list of substitutions *)
let lst_subs_dbg (subs : t_subst list) (size : string) :  ((Types_unify.term * Types_unify.term) list list) =
  let rec help cnt str = function
    | sub::subs -> help (cnt+1) ((lst_sub_dbg sub)::str) subs
    | []    -> str
  in
  if subs = [] then [] else help 1 [] subs
;;

(* ***** *)

(* returns a string describing an equation system for xor *)
let str_eqns_dbg table (eqns : term list list) : string =
  let rec help str = function
    | (ts)::tail -> help (str^"0 = "^(str_term_dbg table (Xor ts))^"\n") tail
    | []         -> str
  in
  if eqns = [] then "empty eqns\n" else help "" eqns
;;


(* returns a string representation of a partial order po *)
let str_po_dbg table (po : t_po) : string =
  let rec help str = function
    | (n,ls)::tail -> help (str^(str_term_dbg table (Var n))^" < "^(str_list_dbg table ls)^"\n") tail
    | []           -> str
  in
  if po = [] then "empty po\n" else help "" po
;;
